package shapes;
//import java.awt.Shape;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Point;
import util.models.AListenable;
import util.models.Listenable;

import java.awt.*;
import java.awt.geom.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.rmi.RemoteException;
public abstract class ShapeModel extends AListenable
      implements RemoteShape, java.io.Serializable, Cloneable, Comparable<RemoteShape>
{
	public final static int DEFAULT_X = 0;
	public final static int DEFAULT_Y = 0;
	public final static int DEFAULT_WIDTH = 20;
	public final static int DEFAULT_HEIGHT = 30;
	int zIndex;
    private Rectangle bounds;
    boolean threeD;
    boolean raised;
    boolean rounded;
    Stroke basicStroke;
    Paint gradientPaint;
    
    public ShapeModel (Rectangle theBounds)throws RemoteException 
    {
        super();
        //bounds = theBounds;
		bounds = new Rectangle(theBounds.x, theBounds.y, theBounds.width, theBounds.height);
    }
    void init (Rectangle theBounds) {
 	   bounds = theBounds;
     }
	public ShapeModel (int x, int y, int width, int height)throws RemoteException 
    {
        super();
        bounds = new Rectangle(x, y, width, height);
    }
	public ShapeModel ()throws RemoteException 
    {
		this(new Rectangle(DEFAULT_X, DEFAULT_Y, DEFAULT_WIDTH, DEFAULT_HEIGHT));
    }	
	
    public Rectangle getBounds()
    {
        return (bounds);
    }
    public void setBounds(int x, int y, int width, int height)
    {
        //System.out.println("Set Bounds called " + x + " " + y + " " + width + " " + height);
		bounds.setBounds(x, y, width, height);
        //this.setChanged();
     	//this.notifyObservers();
     	this.notifyListeners();
    }
	
	public void setBounds(Rectangle r)
    {
        //System.out.println("Set Bounds called " + x + " " + y + " " + width + " " + height);
		setBounds(r.x, r.y, r.width, r.height);
        //this.setChanged();
     	//this.notifyObservers();
     	//this.notifyListeners();
    }
	public void setBounds(Point northWest, Point southEast)
    {
        
		setBounds(northWest.x, northWest.y, southEast.x - northWest.x, southEast.y - northWest.y);
        //this.setChanged();
     	//this.notifyObservers();
     	//this.notifyListeners();
    }
	public void setSize(Dimension d)
    {
		bounds.setSize(d);
        //this.setChanged();
     	//this.notifyObservers();
     	//this.notifyListeners();
    }
	public void setSize(int width, int height)
    {		
		bounds.setSize(new Dimension(width, height));
        //this.setChanged();
     	//this.notifyObservers();
     	this.notifyListeners();
    }
	public void setWidth(int width)
    {
		if (bounds.getWidth() == width)
			return;
		bounds.setSize(new Dimension(width, getBounds().height));
        //this.setChanged();
     	//this.notifyObservers();
     	this.notifyListeners();
    }
	public void setHeight(int height)
    {
		if (bounds.getHeight() == height)
			return;
		bounds.setSize(new Dimension(getBounds().width, height));
        //this.setChanged();
     	//this.notifyObservers();
     	this.notifyListeners();
    }
	public Dimension getSize()
    {
		return bounds.getSize();
        //this.setChanged();
     	//this.notifyObservers();
     	//this.notifyListeners();
    }
	public int getX() {
		return bounds.x;
	}
	public int getY() {
		return bounds.y;
	}
	public void setX(int newVal) {
		bounds.x = newVal;
		this.notifyListeners();
	}
	public void setY(int newVal) {
		bounds.y = newVal;
		this.notifyListeners();
	}
	public void setPosition(Point p) {
		bounds.x = p.x;
		bounds.y = p.y;
		this.notifyListeners();
	}
	public Point getPosition() {
		return new Point(bounds.x, bounds.y);
	}
	public int getHeight() {
		return getBounds().height;
	}
	public int getWidth() {
		return getBounds().width;
	}
	public Point getNWCorner()
    {
		return new Point(bounds.x, bounds.y);
    }
	public void setNWCorner(int x, int y)
    {
		bounds.x = x;
		bounds.y = y;
		this.notifyListeners();
    }
	public void setNWCorner(Point p)
    {
		setNWCorner(p.x, p.y);
    }
	public Point getCenter()
    {
		return centerFromRectangle(bounds);
    }
	public void setCenter(Point center)
    {
		this.setBounds(rectangleFromCenter(center, bounds.width, bounds.height));
    }
	public void setCenter(int x, int y ) 
    {
		setCenter(new Point(x,y));
    }
	public static Point centerFromRectangle (Rectangle r) {
		return new Point(r.x + r.width/2, r.y + r.height/2);
	}
	public static Rectangle rectangleFromCenter(Point center, int width, int height) {
		return new Rectangle(center.x - width/2, center.y - height/2, width, height);
		
	}
	
	public Point getNECorner()
    {
		return nECornerFromRectangle(bounds);
    }
	public void setNECorner(Point p)
    {
		this.setBounds(rectangleFromNECorner(p, bounds.width, bounds.height));
    }
	public void setNECorner(int x, int y ) 
    {
		setNECorner(new Point(x,y));
    }
	public static Point nECornerFromRectangle (Rectangle r) {
		return new Point(r.x + r.width, r.y);
	}
	public static Rectangle rectangleFromNECorner(Point p, int width, int height) {
		return new Rectangle(p.x - width, p.y, width, height);
		
	}
	public Point getSWCorner()
    {
		return sWCornerFromRectangle(bounds);
    }
	public void setSWCorner(Point p)
    {
		this.setBounds(rectangleFromSWCorner(p, bounds.width, bounds.height));
    }
	public void setSWCorner(int x, int y ) 
    {
		setSWCorner(new Point(x,y));
    }
	public static Point sWCornerFromRectangle (Rectangle r) {
		return new Point(r.x, r.y + r.height);
	}
	public static Rectangle rectangleFromSWCorner(Point p, int width, int height) {
		return new Rectangle(p.x, p.y - height, width, height);
		
	}
	public Point getSECorner()
    {
		return sECornerFromRectangle(bounds);
    }
	public void setSECorner(Point p)
    {
		this.setBounds(rectangleFromSECorner(p, bounds.width, bounds.height));
    }
	public void setSECorner(int x, int y ) 
    {
		setSECorner(new Point(x,y));
    }
	public static Point sECornerFromRectangle (Rectangle r) {
		return new Point(r.x + r.width, r.y + r.height);
	}
	public static Rectangle rectangleFromSECorner(Point p, int width, int height) {
		return new Rectangle(p.x - width, p.y - height, width, height);
		
	}
	public Point getNorthEnd()
    {
		return northEndFromRectangle(bounds);
    }
	public void setNorthEnd(Point p)
    {
		this.setBounds(rectangleFromNorthEnd(p, bounds.width, bounds.height));
    }
	public void setNorthEnd(int x, int y ) 
    {
		setSECorner(new Point(x,y));
    }
	public static Point northEndFromRectangle (Rectangle r) {
		return new Point(r.x + r.width/2, r.y);
	}
	public static Rectangle rectangleFromNorthEnd(Point p, int width, int height) {
		return new Rectangle(p.x - width/2, p.y, width, height);
		
	}
	public Point getSouthEnd()
    {
		return southEndFromRectangle(bounds);
    }
	public void setSouthEnd(Point p)
    {
		this.setBounds(rectangleFromSouthEnd(p, bounds.width, bounds.height));
    }
	public void setSouthEnd(int x, int y ) 
    {
		setSouthEnd(new Point(x,y));
    }
	public static Point southEndFromRectangle (Rectangle r) {
		return new Point(r.x + r.width/2, r.y + r.height);
	}
	public static Rectangle rectangleFromSouthEnd(Point p, int width, int height) {
		return new Rectangle(p.x + width/2, p.y - height, width, height);
		
	}
	public Point getWestEnd()
    {
		return westEndFromRectangle(bounds);
    }
	public void setWestEnd(Point p)
    {
		this.setBounds(rectangleFromWestEnd(p, bounds.width, bounds.height));
    }
	public void setWestEnd(int x, int y ) 
    {
		setWestEnd(new Point(x,y));
    }
	public static Point westEndFromRectangle (Rectangle r) {
		return new Point(r.x, r.y + r.height/2);
	}
	public static Rectangle rectangleFromWestEnd(Point p, int width, int height) {
		return new Rectangle(p.x , p.y - height/2, width, height);
		
	}
	public Point getEastEnd()
    {
		return eastEndFromRectangle(bounds);
    }
	public void setEastEnd(Point p)
    {
		this.setBounds(rectangleFromEastEnd(p, bounds.width, bounds.height));
    }
	public void setEastEnd(int x, int y ) 
    {
		setEastEnd(new Point(x,y));
    }
	public static Point eastEndFromRectangle (Rectangle r) {
		return new Point(r.x + r.width, r.y + r.height/2);
	}
	public static Rectangle rectangleFromEastEnd(Point p, int width, int height) {
		return new Rectangle(p.x - width , p.y - height/2, width, height);
		
	}
	Color color;
	public Color getColor() {
		return color;
	}
	public void  setColor(Color newVal) {
		color = newVal;
	}	
	boolean isFilled = false;
	public boolean isFilled() {
		return isFilled;
	}
	public void  setFilled(boolean newVal) {
		isFilled = newVal;
	}
	
	
	boolean is3DRecangle = false;
	public boolean is3DRecangle() {
		return is3DRecangle;
	}
	public void  set3DRecangle(boolean newVal) {
		is3DRecangle = newVal;
	}
	
	int arcWidth, arcHeight;
	
	
	Font font;
	public Font getFont() {
		return font;
	}
	public void  setFont(Font newVal) {
		font = newVal;
	}	
	
    public Object clone()
    {
        ShapeModel clone = null;
        try
        {
            clone = (ShapeModel) super.clone();
            Rectangle bounds = this.getBounds();
            clone.bounds = new Rectangle(bounds);
			//clone.setBounds(new Rectangle(bounds));
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            return (clone);
        }
    }
    public Object remoteClone()
    {
        return clone();
    }
     private void writeObject(java.io.ObjectOutputStream out)
        throws java.io.IOException
    {
        out.defaultWriteObject();
    }

    private void readObject(java.io.ObjectInputStream in)
        throws java.io.IOException, ClassNotFoundException
    {
        in.defaultReadObject();
    }
    public boolean contains(double x, double y) {
    	return getBounds().contains(x, y);
		}
    public boolean contains(double x, double y, double w, double h) { 
    //Tests if the interior of the Shape entirely contains the specified rectangular area.
	return getBounds().contains (x, y, w, h);
}
public boolean contains(Point2D p) {
    //Tests if a specified Point2D is inside the boundary of the Shape. 
	return getBounds().contains(p);
	//return false;
}
public boolean contains(Rectangle2D r) {
    //Tests if the interior of the Shape entirely contains the specified Rectangle2D. 
	return getBounds().contains(r);
}

public Rectangle2D getBounds2D() { 
    //Returns a high precision and more accurate bounding box of the Shape than the getBounds method. 
	return getBounds().getBounds2D();
}
public PathIterator getPathIterator(AffineTransform at) {
    //Returns an iterator object that iterates along the Shape boundary and provides access to the geometry of the Shape outline. 
	return getBounds().getPathIterator(at);
}
 public PathIterator getPathIterator(AffineTransform at, double flatness) {
    //Returns an iterator object that iterates along the Shape boundary and provides access to a flattened view of the Shape outline geometry. 
	 return getBounds().getPathIterator(at, flatness);
}
public boolean intersects(double x, double y, double w, double h) {
    //Tests if the interior of the Shape intersects the interior of a specified rectangular area. 
	return getBounds().intersects(x,y,w,h);
}
public boolean intersects(Rectangle2D r) {
	return getBounds().intersects(r);
}
public int getZIndex() throws RemoteException {
	return zIndex;
}
public void setZIndex(int newValue)throws RemoteException {
	zIndex = newValue;
	this.notifyListeners(true);
}
public static int compare(RemoteShape s1, RemoteShape s2) {
	try {
	if (s1.getZIndex() < s2.getZIndex())
		return - 1;
	else if (s1.getZIndex() == s2.getZIndex())
		return 0;
	else
		return 1;
	} catch (Exception e) {
		return 0;
	}
	
}

public int compareTo(RemoteShape s2) {
	return compare(this, s2);
}
@Override
public boolean isThreeD() throws RemoteException {
	return threeD;
}
@Override
public void setThreeD(boolean threeD) throws RemoteException {
	this.threeD = threeD;
}
@Override
public boolean isRaised() throws RemoteException {
	return raised;
}
@Override
public void setRaised(boolean raised) throws RemoteException {
	this.raised = raised;
}

@Override
public boolean isRounded() throws RemoteException {
	return rounded;
}
@Override
public void setRounded(boolean rounded) throws RemoteException {
	this.rounded = rounded;
}

@Override
public Stroke getStroke() throws RemoteException {
	return basicStroke;
}
@Override
public void setStroke(Stroke basicStroke) throws RemoteException {
	this.basicStroke = basicStroke;
}
@Override
public Paint getPaint() throws RemoteException {
	return gradientPaint;
}
@Override
public void setPaint(Paint gradientpaint) throws RemoteException {
	this.gradientPaint = gradientpaint;
}
@Override
public int getArcWidth() {
	return arcWidth;
}
@Override
public void setArcWidth(int arcWidth) {
	this.arcWidth = arcWidth;
}
@Override
public int getArcHeight() {
	return arcHeight;
}
@Override
public void setArcHeight(int arcHeight) {
	this.arcHeight = arcHeight;
}


}
